#include <Wire.h>
#include <TroykaRTC.h>
#include <Adafruit_NeoPixel.h>

#define LEN_TIME 12
#define LEN_DATE 12
#define LEN_DOW 12

RTC clock;
Adafruit_NeoPixel pixels(16, 9, NEO_GRB + NEO_KHZ800);

String time;
String date;
String weekDay;

// -2 - по экрану пробегает белая точка
// -1 - гасим экран
// 0 - зажигаем зеленый экран
// x > 0 - зажигаем красный экран с x яркими точками
void error(char e) {
  pixels.clear();
  switch (e) {
    case -2:
      for (byte i = 0; i < 16; i++) {
        pixels.clear();
        pixels.setPixelColor(i, pixels.Color(3, 3, 3));
        pixels.show();
        delay(75);
      }
      break;
    case -1: break;
    case 0:
      for (byte i = 0; i < 16; i++)
        pixels.setPixelColor(i, pixels.Color(0, 3, 0));
      break;
    default:
      for (byte i = 0; i < 16; i++)
        pixels.setPixelColor(i, pixels.Color(3, 0, 0));

      for (byte i = 0; i < e; i++)
        pixels.setPixelColor(i, pixels.Color(100, 0, 0));
      break;
  }
  pixels.show();
}

word test_clock(void) {
  //устанавливаем время
  clock.set(1, 1, 1, 1, 1, 2000, MONDAY);
  clock.read();
  clock.getTimeStamp(time, date, weekDay);

  Serial.print(time);
  Serial.print("\t");
  Serial.print(date);
  Serial.print("\t");
  Serial.println(weekDay);
  
  if (time != "01:01:01") {
    return 2; // часы не ходят!
  }

  error(-2); // в течении чуть более 1 секунды бежит белая точка
  error(-2);
  delay(100);

  
  clock.read();
  clock.getTimeStamp(time, date, weekDay);
  
  Serial.print(time);
  Serial.print("\t");
  Serial.print(date);
  Serial.print("\t");
  Serial.println(weekDay);

  if (!( date == "01.01.2000" && (time == "01:01:03" || time == "01:01:02"))) {
    return 3;
  }

  return 0;
}

bool test_i2c(byte address) {
  Wire.beginTransmission(address);
  byte error = Wire.endTransmission();
  return error == 0 ? true : false;
}


void setup() {
  Serial.begin(9600);
  pinMode(10, INPUT); // сенсорная кнопка

  pixels.begin();
  clock.begin();
  error(-2); // по экрану пробежит белая точка
  error(-1); // потом погасим экран
}

void loop() {
  if (digitalRead(10) == 1) { // ждем нажатия на кнопку
    while (digitalRead(10) == 1); // сработает по отпусканию кнопки

    error(-1); // гасим экран


    error(1); // если не видим EEPROM (а ведь она-то точно есть) - значит проблемы с шиной i2c - 1 яркая красная
    if (!test_i2c(0x50)) {
      return;
    }
    error(-1); // гасим экран

    if (!test_i2c(0x68)) {
      error(2);  // если не видим RTC - 2 яркие красные
      return;
    }

    byte e = test_clock(); // если видим часы, можно попробовать с ними пообщаться

    // если часы не инициализировались - 3 яркие красные
    // если не часы "не тикают" (кварц) - 4 яркие красные
    // если есть проблемы записи/чтения - 5 ярких красных
    if (e > 0) {
      error(e + 2);
      return;
    }
    error(0); // все хорошо - зеленый
  }
  delay(20);
}
